﻿Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters
Imports System.Net.Sockets
Imports System.IO
Public Class Client

#Region " Variables "

    ''' <summary>
    ''' IP address
    ''' </summary>
    ''' <remarks></remarks>
    Private PublicIP As String = String.Empty

    ''' <summary>
    ''' The infinite reading thread.
    ''' </summary>
    ''' <remarks></remarks>
    Private readThread As Threading.Thread 'Reader thread.

    ''' <summary>
    ''' The Connected Client.
    ''' </summary>
    ''' <remarks></remarks>
    Private m_Client As TcpClient 'the tcp client, do not ever dispose this!

    ''' <summary>
    ''' Formatter for Handling queries.
    ''' </summary>
    ''' <remarks></remarks>
    Private BinFormatter As Binary.BinaryFormatter 'The binary formatter that handles our queries.

    ''' <summary>
    ''' Our Timer for checking for timeouts.
    ''' </summary>
    ''' <remarks></remarks>
    Private WithEvents Timer As New System.Timers.Timer With {.AutoReset = True, .Interval = 9000, .Enabled = False}

#End Region

#Region " Properties "

    Public Property Username As String = String.Empty
    Public Property Loggedin As Boolean = False

#End Region

#Region " Events "

    ''' <summary>
    ''' The Message Received Event.
    ''' </summary>
    ''' <param name="cli">The Client that sent the message.</param>
    ''' <param name="obj">The Object the client sent.</param>
    ''' <remarks></remarks>
    Friend Event Message(ByVal cli As Client, ByVal obj As Object)

    ''' <summary>
    ''' The Error Event for a Client.
    ''' </summary>
    ''' <param name="cli">The Client that threw the error.</param>
    ''' <param name="err">The exception data.</param>
    ''' <remarks></remarks>
    Friend Event ErrorOut(ByVal cli As Client, ByVal err As Exception)

#End Region

#Region " Subs "

    ''' <summary>
    ''' Create a new Client.
    ''' </summary>
    ''' <param name="client">The TCP Client that's connecting.</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal client As TcpClient)
        m_Client = client 'Transfer over the TcpClient to this class.
        m_Client.SendBufferSize = 1400
        m_Client.ReceiveBufferSize = 1400
        readThread = New System.Threading.Thread(AddressOf client_doRead) 'create new reading thread.
        readThread.IsBackground = True 'set it to the background to not disturb the server
        readThread.Start() 'start it.
        Timer.Enabled = True 'create our timeout timer.
        Timer.Start() 'Start the timer.
    End Sub

    ''' <summary>
    ''' The infinite doRead() sub. Handles all reading from the client.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub client_doRead()
        Try
            Do
                BinFormatter = New Binary.BinaryFormatter()
                'create a generic object
                Dim obj As New Object
                'deserialize the stream when a message comes in.
                obj = BinFormatter.Deserialize(m_Client.GetStream)
                RaiseEvent Message(Me, obj)
            Loop
        Catch ex As Threading.ThreadAbortException
            'Thread aborting. That's OK. I'm game with this.
            Debug.WriteLine("Threading Exception, probably aborted.")
        Catch ex As SerializationException
            Debug.WriteLine("Serialization Error, that's not good.")
            RaiseEvent ErrorOut(Me, ex)
            Me.Dispose()
        Catch ex As IOException
            Debug.WriteLine("IO Exception. Stream died.")
            RaiseEvent ErrorOut(Me, ex)
            Me.Dispose()
        Catch ex As Exception
            Debug.WriteLine("General Exception")
            RaiseEvent ErrorOut(Me, ex)
            Me.Dispose()
        End Try
    End Sub

    ''' <summary>
    ''' Send a Serialized Message to the Client.
    ''' </summary>
    ''' <param name="message">The Object to send.</param>
    ''' <remarks></remarks>
    Public Sub SendMessage(ByVal message As Object)
        Dim SMAT As New Threading.Thread(New Threading.ParameterizedThreadStart(AddressOf SendMessageAsync))
        SMAT.Start(message)
    End Sub

    ''' <summary>
    ''' Sends a message across the stream Asynchronously.
    ''' </summary>
    ''' <param name="message">The message to send.</param>
    ''' <remarks></remarks>
    Private Sub SendMessageAsync(ByVal message As Object)
        Try
            SyncLock m_Client.GetStream
                BinFormatter = New Binary.BinaryFormatter()
                BinFormatter.Serialize(m_Client.GetStream, message)
            End SyncLock
            Timer.Stop()
            Timer.Start()
        Catch BSE As SerializationException
            Debug.WriteLine(BSE.ToString)
            Me.Dispose()
        Catch exi As InvalidOperationException
            'Debug.WriteLine(exi.StackTrace)
            'Debug.WriteLine(exi.Data.Item(0).ToString)
            Me.Dispose()
        Catch ex As Exception
            'The stream is broken.
            Debug.WriteLine(ex.ToString)
            Me.Dispose()
        End Try
    End Sub

    ''' <summary>
    ''' Dispose of the Client
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Dispose()
        Me.VariableReset()
        RaiseEvent ErrorOut(Me, New Exception)
    End Sub

    ''' <summary>
    ''' Resets a few variables to nothing.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub VariableReset()
        Timer.Stop()
        Timer.Dispose()
        BinFormatter = Nothing
        Try
            readThread.Abort()
        Catch
        End Try
        m_Client.Close()
        m_Client.Client.Close()
        GC.Collect()
    End Sub

    ''' <summary>
    ''' Returns a String Representation of the Client.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Overrides Function ToString() As String
        Return Me.PublicIPAddress
    End Function

    ''' <summary>
    ''' Returns the IP address of the Client.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property PublicIPAddress() As String
        Get
            If PublicIP = String.Empty Then
                Try
                    ' Get the clients IP address using Client property            

                    Dim ipend As Net.IPEndPoint = CType(m_Client.Client.RemoteEndPoint, Net.IPEndPoint)
                    If Not ipend Is Nothing Then
                        PublicIP = ipend.Address.ToString
                    End If
                Catch ex As System.ObjectDisposedException
                    PublicIP = String.Empty
                Catch ex As SocketException
                    PublicIP = String.Empty
                End Try
            End If
            Return PublicIP
        End Get
    End Property

    Private Sub Timer_Elapsed(ByVal sender As Object, ByVal e As System.Timers.ElapsedEventArgs) Handles Timer.Elapsed
        Me.SendMessage(New DLL.MessageStructure With {.Message = "5"})
    End Sub

#End Region

End Class