﻿Option Strict Off
Imports System.Text
Imports System.IO
Imports PokeHack
Imports DLL

Public Class PokemonClass

    Public ReadOnly Property PokemonDataLocation As String
        Get
            Return IO.Path.Combine(Application.StartupPath, "UploadedFiles")
        End Get
    End Property
    Public ReadOnly Property PokemonFileLocation As String
        Get
            Return IO.Path.Combine(Application.StartupPath, "TextFiles")
        End Get
    End Property
    Public ReadOnly Property CustomPokemonFileLocation As String
        Get
            Return IO.Path.Combine(Application.StartupPath, "CustomTextFiles")
        End Get
    End Property

    Public Function MD5CalcFile(ByVal PokemonStream As IO.Stream) As String
        ' open file (as read-only)
        Using md5 As New System.Security.Cryptography.MD5CryptoServiceProvider
            ' hash contents of this stream
            Dim hash() As Byte = md5.ComputeHash(PokemonStream)
            ' return formatted hash
            Return ByteArrayToString(hash)
        End Using
    End Function

    ' utility function to convert a byte array into a hex string
    Private Function ByteArrayToString(ByVal arrInput() As Byte) As String
        Dim sb As New System.Text.StringBuilder(arrInput.Length * 2)
        For i As Integer = 0 To arrInput.Length - 1
            sb.Append(arrInput(i).ToString("X2"))
        Next
        Return sb.ToString().ToLower
    End Function

    ''' <summary>
    ''' Looks to see if an ID has been created already.
    ''' </summary>
    ''' <param name="ID">The ID to look for.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function CheckForExistingData(ByVal ID As String) As Boolean
        Dim List As New List(Of String)
        List.AddRange(IO.Directory.GetFiles(PokemonFileLocation, "*.txt"))
        List.AddRange(IO.Directory.GetFiles(CustomPokemonFileLocation, "*.txt"))
        For Each item As String In List
            If New PokemonInformation(item).ID = ID Then Return True
        Next
        Return False
    End Function

    ''' <summary>
    ''' Returns the direct path to the PKM file.
    ''' </summary>
    ''' <param name="ID">The ID.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ReturnFilePath(ByVal ID As String) As String
        Dim List As New List(Of String)
        List.AddRange(IO.Directory.GetFiles(PokemonFileLocation, "*.txt"))
        List.AddRange(IO.Directory.GetFiles(CustomPokemonFileLocation, "*.txt"))
        For Each item As String In List
            Dim PkmInfo As New PokemonInformation(item)
            If PkmInfo.ID = ID Then Return PkmInfo.Path
        Next
        Return String.Empty
    End Function

    Public Function ReturnTextPath(ByVal ID As String) As String
        Dim List As New List(Of String)
        List.AddRange(IO.Directory.GetFiles(PokemonFileLocation, "*.txt"))
        List.AddRange(IO.Directory.GetFiles(CustomPokemonFileLocation, "*.txt"))
        For Each item As String In List
            Dim PkmInfo As New PokemonInformation(item)
            If PkmInfo.ID = ID Then Return item
        Next
        Return String.Empty
    End Function

    ''' <summary>
    ''' Creates a Custom Pokemon File.
    ''' </summary>
    ''' <param name="data">Pokemon Data</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function CreatePokemonData(ByVal data As PokemonData) As Boolean

        'First create the .pkm file.
        Dim Filename As String = String.Format("{0}_{1}.pkm", New RandomStringMaker().GenerateString(New Random().Next(5, 9)), data.PokemonName)
        'Data path for customs
        Dim PkmDataPath As String = IO.Path.Combine(PokemonDataLocation, Filename)
        'Text file location
        Dim TxtDataPath As String = IO.Path.Combine(CustomPokemonFileLocation, String.Format("{0}.txt", data.ID))

        If CheckForExistingData(data.ID) = False AndAlso Not File.Exists(TxtDataPath) AndAlso Not New PokeHack.Pokemon(data.PokemonData).IsEgg Then
            'Check for encryption.
            If PokeHack.PokeHack.IsEncrypted(New Pokemon(data.PokemonData)) Then data.PokemonData = PokeHack.PokeHack.Decrypt(data.PokemonData).GetRawData
            'Check if it's a party or PC.
            If data.PokemonData.Length < 236 Then data.PokemonData = PartyPokemon.ConvertPCToParty(data.PokemonData).GetRawData()
            'write to file.
            IO.File.WriteAllBytes(PkmDataPath, data.PokemonData)
        Else
            Return False
        End If
        Try
            'now create the associated text file.
            Using SW As New IO.StreamWriter(TxtDataPath)
                'Name
                SW.WriteLine(data.PokemonName.ToLower)
                'ID
                SW.WriteLine(data.ID)
                'Path
                SW.WriteLine(PkmDataPath)
                'IP
                SW.WriteLine(data.IP)
                'Internal name
                SW.WriteLine(ReturnInternalName(data.PokemonData))
            End Using
            'Return success.
            Return True

        Catch ex As Exception
            If File.Exists(PkmDataPath) Then File.Delete(PkmDataPath)
            If File.Exists(TxtDataPath) Then File.Delete(TxtDataPath)
            Return False
        End Try
    End Function

    Public Function ReturnInternalName(ByVal bytes() As Byte) As String
        Dim pkm As New Pokemon(bytes)
        Return pkm.Name
    End Function

    Public Function ReturnInternalName(ByVal path As String) As String
        If File.Exists(path) Then Return ReturnInternalName(IO.File.ReadAllBytes(path)) Else Return String.Empty
    End Function

    ''' <summary>
    ''' Returns the uploader's IP
    ''' </summary>
    ''' <param name="ID">The ID</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ReturnUploaderIP(ByVal ID As String) As String
        Dim List As New List(Of String)
        List.AddRange(IO.Directory.GetFiles(PokemonFileLocation, "*.txt"))
        List.AddRange(IO.Directory.GetFiles(CustomPokemonFileLocation, "*.txt"))
        For Each item As String In List
            Dim PkmInfo As New PokemonInformation(item)
            If PkmInfo.ID = ID Then Return PkmInfo.IP
        Next
        Return String.Empty
    End Function

End Class

Public Class RandomStringMaker
    Dim ListChars As New List(Of String)({"a", "b", "c", "d", "e", "f", "g", "h", "i", "j", _
                             "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", _
                             "u", "v", "w", "x", "y", "z", _
                             "A", "B", "C", "D", "E", "F", "G", "H", "I", "J", _
                             "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", _
                             "U", "V", "W", "X", "Y", "Z", _
                             "0", "1", "2", "3", "4", "5", "6", "7", "8", "9"})
    Dim History As New List(Of String)

    Public Function GenerateString(ByVal length As Integer) As String
        Dim RandNumb As New Random
        Dim StrBld As New StringBuilder

        If length > 0 Then
            StrBld.Append(ListChars(RandNumb.Next(0, ListChars.Count - 1)))
            Do Until Not History.Contains(StrBld.ToString) AndAlso StrBld.Length >= length
                For i As Integer = 1 To length - 1
                    StrBld.Append(ListChars(RandNumb.Next(0, ListChars.Count - 1)))
                Next
            Loop
            History.Add(StrBld.ToString)
            Return StrBld.ToString
        Else
            Return String.Empty
        End If
    End Function

End Class

Public Structure PokemonInformation
    Public Property Name As String
    Public Property ID As String
    Public Property Path As String
    Public Property IP As String
    Public Property IName As String
    Public Sub New(ByVal file As String)
        Using SR As New IO.StreamReader(file)
            Name = SR.ReadLine()
            ID = SR.ReadLine()
            Path = SR.ReadLine()
            IP = SR.ReadLine()
            IName = SR.ReadLine()
        End Using
    End Sub
End Structure