﻿Public Class ReservationHandler
    Public Property RequestPath As String = IO.Path.Combine(Application.StartupPath, "UserRequests")
    Public Property PokemonText As String = IO.Path.Combine(Application.StartupPath, "TextFiles")

    ''' <summary>
    ''' Checks for a Reservation.
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function CheckForReservation(ByVal IP As String, ByVal Override As Boolean) As Boolean
        If Override Then Return False
        For Each file As String In IO.Directory.GetFiles(RequestPath, "*.txt")
            If IO.Path.GetFileNameWithoutExtension(file) = IP Then Return True
        Next
        Return False
    End Function

    ''' <summary>
    ''' Looks for a specific ID of an uploaded Pokemon.
    ''' </summary>
    ''' <param name="ID">The MD5/ID of the Pokemon File.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function LoadSpecificID(ByVal ID As String) As PokemonInformation
        For Each file As String In IO.Directory.GetFiles(PokemonText, "*.txt")
            Dim RD As New PokemonInformation(file)
            If RD.ID = ID Then Return RD
        Next
        Return Nothing
    End Function

    ''' <summary>
    ''' Creates a Reservation for an IP.
    ''' </summary>
    ''' <param name="IP">The User's IP Address.</param>
    ''' <param name="ID">The ID to reserve.</param>
    ''' <param name="Override">Overrides a previous reservation.</param>
    ''' <remarks></remarks>
    Public Sub CreateReservation(ByVal IP As String, ByVal ID As String, ByVal Override As Boolean)
        If Not CheckForReservation(IP, Override) Then
            Dim FP As String = IO.Path.Combine(RequestPath, String.Format("{0}.txt", IP))
            If Override Then If IO.File.Exists(FP) Then IO.File.Delete(FP)
            Using SW As New IO.StreamWriter(FP)
                SW.WriteLine(ID)
            End Using
        End If
    End Sub

    ''' <summary>
    ''' Removes a Reservation.
    ''' </summary>
    ''' <param name="IP">The IP to remove.</param>
    ''' <remarks></remarks>
    Public Sub RemoveReservation(ByVal IP As String)
        If CheckForReservation(IP, False) Then
            IO.File.Delete(IO.Path.Combine(RequestPath, String.Format("{0}.txt", IP)))
        End If
    End Sub

End Class

