﻿Option Strict Off
Imports DLL

Public Class ServerBackend
    Public Reports As New List(Of ReportPokemon)
    Private AcceptingThread As New System.Threading.Thread(AddressOf ListeningThread) With {.IsBackground = True}
    Private IncomingClient As Net.Sockets.TcpClient
    Private Listener As Net.Sockets.TcpListener
    Private ReservationClass As New ReservationHandler
    Private PokemonHandler As New PokemonClass
    Private SearchClass As New SearchHandler
    Private Property Port As Integer = 666
    Public clientlist As New List(Of Client)
    Public ReadOnly Property PokemonFileLocation As String
        Get
            Return IO.Path.Combine(Application.StartupPath, "TextFiles")
        End Get
    End Property

#Region " Events "

    Public Event ChatChanged(ByVal user As String, ByVal state As Boolean)

    Public Event Log(ByVal sender As Object, ByVal message As String)

    Protected Overridable Sub OnLogMessage(ByVal message As String)
        RaiseEvent Log(Me, message)
    End Sub

#End Region

    Public Sub New(ByVal listeningPort As Integer)
        Port = listeningPort
        Listener = New Net.Sockets.TcpListener(Net.IPAddress.Any, Port)
        PokeHack.Pokemon.LoadDataFiles()
    End Sub

    Public Sub Begin()
        Me.OnLogMessage("Server Starting")
        Listener.Start()
        AcceptingThread.Start()
        Me.OnLogMessage(String.Format("Search Class has {0} items in the list.", SearchClass.List.Count))
    End Sub

    Private Sub ListeningThread()
        Try
            Do
                IncomingClient = Listener.AcceptTcpClient
                Me.OnLogMessage("Connection Established")
                Dim Cli As New Client(IncomingClient)
                Me.OnLogMessage(String.Format("Connection From IP: {0}", Cli.PublicIPAddress))
                AddHandler Cli.Message, AddressOf ClientMessages
                AddHandler Cli.ErrorOut, AddressOf ClientError
                Me.OnLogMessage(String.Format("Client {0} connected!", Cli.PublicIPAddress))
                clientlist.Add(Cli)
            Loop
        Catch ex As Exception
            Me.OnLogMessage(ex.Message.ToString)
        End Try
    End Sub

    Private Sub ThreadedMessageHandler(ByVal td As ThreadData)
        Me.OnLogMessage(String.Format("Incoming message from {0}", td.Cli.PublicIPAddress))

        If td.Obj.GetType Is GetType(ReportPokemon) Then
            Dim RP As ReportPokemon = CType(td.Obj, ReportPokemon)
            RP.ReportedBy = td.Cli.PublicIPAddress
            Reports.Add(RP)
            Me.OnLogMessage(String.Format("Report filed by {0} on ID {1}", RP.ReportedBy, RP.ID))
        End If

        If td.Obj.GetType Is GetType(ReservePokemon) Then
            Dim RP As ReservePokemon = CType(td.Obj, ReservePokemon)
            Me.OnLogMessage(String.Format("{0} is reserving a Pokemon with the ID: {1}", td.Cli.PublicIPAddress, RP.ID))
            RP.IP = td.Cli.PublicIPAddress
            If Not ReservationClass.CheckForReservation(RP.IP, RP.Overwrite) Then
                ReservationClass.CreateReservation(RP.IP, RP.ID, RP.Overwrite)
                td.Cli.SendMessage(New DLL.MessageStructure With {.Message = "0"})
            Else
                td.Cli.SendMessage(New DLL.MessageStructure With {.Message = "1"})
            End If
        End If

        If td.Obj.GetType Is GetType(DeleteStructure) Then
            ReservationClass.RemoveReservation(td.Cli.PublicIPAddress)
        End If

        If td.Obj.GetType Is GetType(PokemonData) Then
            Me.OnLogMessage(String.Format("{0} is uploading some Pokemon data.", td.Cli.PublicIPAddress))
            Dim PD As PokemonData = CType(td.Obj, PokemonData)
            PD.IP = td.Cli.PublicIPAddress
            If PokemonHandler.CreatePokemonData(PD) Then
                td.Cli.SendMessage(New DLL.MessageStructure With {.Message = "2"})
                Me.OnLogMessage(String.Format("{0}'s Pokemon was unique, accepted!", td.Cli.PublicIPAddress))
                SearchClass.Refresh()
            Else
                td.Cli.SendMessage(New DLL.MessageStructure With {.Message = "3"})
                Me.OnLogMessage(String.Format("{0}'s Pokemon wasn't unique, denied!", td.Cli.PublicIPAddress))
            End If
        End If

        If td.Obj.GetType Is GetType(RequestData) Then
            Dim SD As RequestData = CType(td.Obj, RequestData)
            Me.OnLogMessage(String.Format("{0} is performing a search.", td.Cli.PublicIPAddress))
            Select Case SD.Search
                Case True
                    'Do Specific Search
                    Dim SR As SearchResults = New SearchResults With {.Search = True, .Results = SearchClass.ReturnFilteredList(SD.Name.ToLower)}
                    'Me.OnLogMessage(String.Format("Results for {0}: {1}", td.Cli.PublicIPAddress, SR.Results))
                    td.Cli.SendMessage(SR)
                    'Me.OnLogMessage(String.Format("{0} should have received specific results.", td.Cli.PublicIPAddress))
                Case False
                    'Return full list.

                    'Me.OnLogMessage(String.Format("{0} requested a full Pokemon list.", td.Cli.PublicIPAddress))

                    Dim SR As SearchResults = New SearchResults With {.Search = False, .Results = SearchClass.ReturnUnfilteredList}
                    'Me.OnLogMessage(String.Format("Result is {0} characters long", SR.Results.Length.ToString))
                    td.Cli.SendMessage(SR)
                    'Me.OnLogMessage(String.Format("{0} has received the entire list.", td.Cli.PublicIPAddress))
            End Select
        End If

        If td.Obj.GetType Is GetType(MessageStructure) Then
            Dim MS As MessageStructure = CType(td.Obj, MessageStructure)
            If MS.Message = "0" Then td.Cli.SendMessage(New MessageStructure With {.Message = "5"})
        End If

        If td.Obj.GetType Is GetType(RequestPokemonInformation) Then
            'Convert object
            Dim RPI As RequestPokemonInformation = CType(td.Obj, RequestPokemonInformation)
            'get path
            Dim PokemonLocation As String = PokemonHandler.ReturnFilePath(RPI.ID)
            'create handler
            Dim PokeHackHandler As New PokeHack.Pokemon(IO.File.ReadAllBytes(PokemonLocation))
            'create information
            Dim Info As New Information
            Me.OnLogMessage(String.Format("{0} has requested additional information for ID {1}", td.Cli.PublicIPAddress, RPI.ID))
            'populate information
            Info.DexNumber = PokeHackHandler.Species
            Info.Exp = PokeHackHandler.Experience
            Info.FateFulEncounter = PokeHackHandler.IsFatefulEncounter
            Info.Gender = PokeHackHandler.Gender.ToString
            Info.InternalID = RPI.ID
            Info.IsEgg = PokeHackHandler.IsEgg
            Info.Level = PokeHackHandler.Level
            Info.Moves = PokeHackHandler.Moves
            Info.Name = PokeHackHandler.Name
            Info.Nature = PokeHackHandler.Nature
            Info.NickName = If(PokeHackHandler.IsNicknamed, PokeHackHandler.Nickname, String.Empty)
            Info.Origin = PokeHackHandler.Origin.ToString
            Info.OTID = PokeHackHandler.OriginalTrainer
            Info.OTName = PokeHackHandler.OTName
            Info.PkmEV = New Information.EV With {.Attack = PokeHackHandler.AttEV, .Defense = PokeHackHandler.DefEV, .HP = PokeHackHandler.HPEV, .Sp_Atk = PokeHackHandler.SPAttEV, .Sp_Def = PokeHackHandler.SPDefEV, .Speed = PokeHackHandler.SpeEV}
            Info.PkmIV = New Information.IV With {.Attack = PokeHackHandler.AttIV, .Defense = PokeHackHandler.DefIV, .HP = PokeHackHandler.HPIV, .Sp_Atk = PokeHackHandler.SPAttIV, .Sp_Def = PokeHackHandler.SPDefIV, .Speed = PokeHackHandler.SpeIV}
            Info.PkmStats = New Information.Stats With {.Attack = PokeHackHandler.Attack, .Defense = PokeHackHandler.Defense, .HP = PokeHackHandler.HP, .Sp_Atk = PokeHackHandler.SpecialAttack, .Sp_Def = PokeHackHandler.SpecialDefense, .Speed = PokeHackHandler.Speed}
            Info.Shiny = PokeHackHandler.IsShiny
            Info.SID = PokeHackHandler.SecretID

            'send
            td.Cli.SendMessage(Info)

        End If

        If td.Obj.GetType Is GetType(DLL.ChatMessage) Then
            SyncLock clientlist
                For i As Integer = clientlist.Count - 1 To 0 Step -1
                    If clientlist(i).Loggedin Then clientlist(i).SendMessage(td.Obj)
                Next
            End SyncLock
        End If

        If td.Obj.GetType Is GetType(DLL.IndividualPerson) Then
            Dim IP As IndividualPerson = CType(td.Obj, IndividualPerson)
            RaiseEvent ChatChanged(IP.person, IP.login)
            Select Case IP.login
                Case True
                    'The sender is joining.
                    td.Cli.Loggedin = True
                    td.Cli.Username = IP.person
                    SyncLock clientlist
                        Dim msg As New ChatMessage With {.message = String.Format("{0} has joined!", IP.person), .sender = "SERVER!0000"}
                        For i As Integer = clientlist.Count - 1 To 0 Step -1
                            If clientlist(i).Loggedin Then clientlist(i).SendMessage(msg) : clientlist(i).SendMessage(IP)
                        Next
                    End SyncLock
                    td.Cli.SendMessage(New DLL.FullList With {.userlist = GenerateChatList()})

                Case False
                    'The sender is leaving.
                    td.Cli.Loggedin = False
                    td.Cli.Username = String.Empty
                    SyncLock clientlist
                        Dim msg As New ChatMessage With {.message = String.Format("{0} has left!", IP.person), .sender = "SERVER!0000"}
                        For i As Integer = clientlist.Count - 1 To 0 Step -1
                            If clientlist(i).Loggedin Then clientlist(i).SendMessage(msg) : clientlist(i).SendMessage(IP)
                        Next
                    End SyncLock
            End Select
        End If



    End Sub

    Private Function GenerateChatList() As List(Of String)
        Dim RL As New List(Of String)
        SyncLock clientlist
            For Each item As Client In clientlist
                If item.Loggedin Then RL.Add(item.Username)
            Next
        End SyncLock
        Return RL
    End Function

    Private Sub ClientMessages(ByVal cli As Client, ByVal obj As Object)
        Dim t As New System.Threading.Thread(New Threading.ParameterizedThreadStart(AddressOf ThreadedMessageHandler))
        t.Start(New ThreadData With {.Cli = cli, .Obj = obj})
    End Sub

    Private Sub ClientError(ByVal cli As Client, ByVal err As Exception)
        'log it and disconnect them.
        RemoveHandler cli.Message, AddressOf ClientMessages
        RemoveHandler cli.ErrorOut, AddressOf ClientError
        Me.OnLogMessage(String.Format("Client {0} has disconnected!", cli.PublicIPAddress))
        clientlist.Remove(cli)
        If cli.Loggedin Then
            SyncLock clientlist
                For i As Integer = clientlist.Count - 1 To 0 Step -1
                    clientlist(i).SendMessage(New IndividualPerson With {.person = cli.Username, .login = False})
                    clientlist(i).SendMessage(New ChatMessage With {.message = String.Format("{0} has left!", cli.Username), .sender = "SERVER!0000"})
                Next
            End SyncLock
            RaiseEvent ChatChanged(cli.Username, False)
        End If
        cli.Dispose()
        GC.Collect()
    End Sub

    Public Sub Dispose()
        Listener.Stop()
        Try
            AcceptingThread.Abort()
        Catch
        End Try
        For i As Integer = clientlist.Count - 1 To 0 Step -1
            clientlist(i).Dispose()
            'clientlist.RemoveAt(i)
        Next
        AcceptingThread = New Threading.Thread(AddressOf ListeningThread)
        Listener = New Net.Sockets.TcpListener(Net.IPAddress.Any, Port)
        Me.OnLogMessage("The Global Distribution Server has shutdown. All clients have been disconnected.")
    End Sub

    Private Structure ThreadData
        Public Property Cli As Client
        Public Property Obj As Object
    End Structure

    Public Sub GenerateTextFiles(ByVal root As String)
        'Generate text files, fuck yeah.
        Me.OnLogMessage("Generating Text files.")
        Dim RMS As New RandomStringMaker
        For Each item As String In IO.Directory.GetFiles(IO.Path.Combine(root, "PokemonFiles"), "*.pkm")
            Dim FS As New IO.FileStream(item, IO.FileMode.Open)
            Dim ID As String = PokemonHandler.MD5CalcFile(FS)
            FS.Close()
            Using SW As New IO.StreamWriter(IO.Path.Combine(IO.Path.Combine(root, "TextFiles"), String.Format("{0}.txt", ID)))
                SW.WriteLine(IO.Path.GetFileNameWithoutExtension(item).ToLower) 'Name
                SW.WriteLine(ID) '"ID"
                SW.WriteLine(item) 'Path
                SW.WriteLine("127.0.0.1") 'root uploader.
                SW.WriteLine(PokemonHandler.ReturnInternalName(item)) 'internal name.
                Me.OnLogMessage(String.Format("Generated Text File for {0}", IO.Path.GetFileNameWithoutExtension(item).ToLower))
            End Using
        Next
        Me.OnLogMessage("Finished creating text files")
    End Sub

    ''' <summary>
    ''' Kicks a user
    ''' </summary>
    ''' <param name="name">The name to kick.</param>
    ''' <remarks></remarks>
    Public Sub KickChat(ByVal name As String)
        For i As Integer = clientlist.Count - 1 To 0 Step -1
            If clientlist(i).Username = name Then clientlist(i).Dispose() : RaiseEvent ChatChanged(name, False)
        Next
    End Sub

    Public Sub RefreshDirectory()
        SearchClass.Refresh()
    End Sub

End Class